% objdall1 - objective function of DAL with L1 regularization
%
% Copyright(c) 2009 Ryota Tomioka
% This software is distributed under the MIT license. See license.txt

function varargout=objdall1(aa, fnc, ww, A, AT, lambda, eta, solver)

m = length(aa);
n = length(ww);
vv = AT(aa)-ww/eta;

Ip = find(vv>lambda);
In = find(vv<-lambda);

[floss, gloss, hloss]=fnc.d(aa, fnc.args{:});

fval = floss+0.5*eta*(sum((-vv(Ip)+lambda).^2) ...
                      +sum((-vv(In)-lambda).^2));


varargout{1}=fval;

if nargout>1
  vth=l1_softth(vv,lambda);
  varargout{2} = gloss+eta*(A(vth));
  if nargout>2
    I = [Ip; In];
    len = length(I);
    F = sparse(I,1:len, ones(1,len),n,len);
    AF = A(F);
    switch(solver)
     case 'cg'
      prec=spdiag(hloss+eta*sum(AF.^2,2));
      varargout{3} = struct('hloss',hloss,'AF',AF,'I',I,'n',n,'prec',prec);
     otherwise
      varargout{3} = spdiag(hloss)+eta*AF*AF';
      % varargout{3} = spdiag(hloss)+eta*(A(:,Ip)*A(:,Ip)'+A(:,In)*A(:,In)');
    end
  end
end

